<?php

class DispletRetsIdxUsers extends DispletRetsIdxPlugin{
	public function register_user_ajax(){
		check_ajax_referer('displet_register_user_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_registration_request'){
			global $displetretsidx_option;
			if (!empty($_POST['name']) && !empty($_POST['email']) && (empty($displetretsidx_option['require_phone']) || !empty($_POST['phone'])) && (empty($displetretsidx_option['require_working_with_realtor']) || empty($displetretsidx_option['include_working_with_realtor']) || !empty($_POST['realtor']))) {
				if (is_email($_POST['email'])) {
					$user_registration_response = self::register_user($_POST['email'], $_POST['name'], $_POST['phone'], $_POST['realtor'], $_POST['url'], $_POST['upstream_url'], $_POST['last_hash']);
					echo $user_registration_response;
				}
				else{
					echo 'Please enter a valid email address.';
				}
			}
			else{
				echo 'Please complete the required fields.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function register_user($email, $name, $phone = '', $realtor = '', $url = '', $upstream_url = '', $last_hash = '') {
		// Create WP User
		$random_password = wp_generate_password(12, false);
		$user_id = wp_create_user($email, $random_password, $email);
		if (is_wp_error($user_id)) {
			return $user_id->get_error_message();
		}
		$names = self::get_first_and_last_name($name);
		wp_update_user(array(
			'ID' => $user_id,
			'role' => 'displet_user',
			'display_name' => $names['first_name']
		));

		// Login user
		$user_info = get_userdata($user_id);
		$creds = array(
			'user_login' => $user_info->user_login,
			'user_password' => $random_password,
			'remember' => true
		);
		$user = wp_signon($creds, false);

		$cron_job_args = array(
			'user_id' => $user_id,
			'login' => $user_info->user_login,
			'password' => $random_password,
			'name' => $name,
			'first_name' => $names['first_name'],
			'last_name' => $names['last_name'],
			'email' => $email,
			'phone' => $phone,
			'realtor' => $realtor,
			'url' => $url,
			'upstream_url' => $upstream_url,
			'last_hash' => $last_hash,
			'listing_agent_email' => $listing_agent_email
		);
		wp_schedule_single_event(time(), 'displetretsidx_new_user_cron_jobs', array($cron_job_args));

		return 'Successful Registration';
	}

	public function new_user_cron_jobs($args){
		extract($args);
		$assigned_agent_id = self::get_new_duty_agent_id($listing_agent_email);
		DispletRetsIdxEmails::send_new_user_registration($email, $name, $phone, $realtor, $url, $login, $password, $assigned_agent_id);
		update_user_meta($user_id, 'displet_lead', true);
		update_user_meta($user_id, 'displet_agent_id', $assigned_agent_id);
		update_user_meta($user_id, 'first_name', $first_name);
		update_user_meta($user_id, 'last_name', $last_name);
		update_user_meta($user_id, 'nickname', $name);
		update_user_meta($user_id, 'displet_phone', $phone);
		update_user_meta($user_id, 'displet_realtor', $realtor);
		update_user_meta($user_id, 'displet_registration_url', $url);
		update_user_meta($user_id, 'displet_registration_upstream_url', $upstream_url);
		update_user_meta($user_id, 'displet_last_hash', $last_hash);
		$zap_user = array(
			'email' => $email,
			'name' => $name,
			'phone' => $phone,
			'registered_at' => $url,
			'came_from' => $upstream_url
		);
		self::send_to_zapier($zap_user, 'registration');
		$api_user_id = DispletRetsIdxUsersApi::create_user($email, $first_name, $last_name);
		$property_id = self::get_property_id_from_url($url);
		DispletRetsIdxUsersApi::update_user_property_views($api_user_id, $property_id);
		update_user_meta($user_id, 'displet_api_user_id', $api_user_id);
	}

	public function login_user_ajax() {
		check_ajax_referer('displet_login_user_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displetretsidx_user_signon'){
			if (!empty($_POST['email']) && !empty($_POST['password'])) {
				$creds = array();
				$creds['user_login'] = $_POST['email'];
				$creds['user_password'] = $_POST['password'];
				$creds['remember'] = true;
				$user = wp_signon($creds, false);
				if (is_wp_error($user)) {
					echo $user->get_error_message();
				}
				else{
					echo 'Successful Login';
				}
			}
			else {
				echo 'Please complete the required fields.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user_ajax() {
		check_ajax_referer('displet_check_user_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_user_check_request'){
			if (!empty($_POST['email'])) {
				$user_exists = self::check_user($_POST['email']);
				if ($user_exists) {
					echo 'User Exists and Logged In';
				}
			}
			else {
				echo 'There is no email account associated with this user.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function check_user($email){
		$user_id = username_exists($email);
		if (!empty($user_id)) {
			wp_set_auth_cookie($user_id, true);
			return true;
		}
		else{
			return false;
		}
	}

	public function check_login_ajax(){
		check_ajax_referer('displet_check_login_nonce');
		if (is_user_logged_in()) {
			echo 'Successful Login';
		}
		else{
			echo 'User Not Logged In';
		}
		die();
	}

	public function get_new_duty_agent_id($listing_agent_email){
		global $displetretsidx_option;
		if (!empty($displetretsidx_option['duty_agent_use_listing_agent_email']) && !empty($listing_agent_email)) {
			$listing_agent_user = get_user_by('email', $listing_agent_email);
		}
		if (!empty($listing_agent_user)) {
			return $listing_agent_user->ID;
		}
		else{
			$duty_agent_array_position = get_option('displet_rets_idx_duty_agent_array_position');
			if (!empty($displetretsidx_option['duty_agent'])) {
				$duty_agents = array();
				foreach ($displetretsidx_option['duty_agent'] as $agent_id => $value) {
					if ($value == 'true'){
						$duty_agents[] = $agent_id;
					}
				}
				if (!empty($duty_agents)) {
					if (!empty($duty_agent_array_position) && intval($duty_agent_array_position) < count($duty_agents)) {
						$i = intval($duty_agent_array_position);
					}
					else{
						$i = 0;
					}
					$duty_agent_array_position = $i;
					$duty_agent_array_position++;
					update_option('displet_rets_idx_duty_agent_array_position', $duty_agent_array_position);
					return $duty_agents[$i];
				}
			}
		}
	}

	public function update_last_login($user_login, $user){
		if (user_can($user->ID, 'displet_save_properties') && !user_can($user->ID, 'manage_options') && !user_can($user->ID, 'displet_view_leads')) {
			$login_timestamps = get_user_meta($user->ID, 'displet_last_login', true);
			if (empty($login_timestamps) || !is_array($login_timestamps)) {
				$login_timestamps = array();
			}
			$last_login = current_time('timestamp');
			$login_timestamps[] = $last_login;
			update_user_meta($user->ID, 'displet_last_login', $last_login);
			update_user_meta($user->ID, 'displet_logins', $login_timestamps);
		}
	}

	public function update_viewed_searches(){
		check_ajax_referer('displet_update_searches_nonce');
		
		if (isset($_POST['action']) && $_POST['action'] == 'displet_update_searches' && !empty($_POST['last_hash'])){
			$user_id = get_current_user_id();
			if ($user_id != 0 && current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
				$user_searches = get_user_meta($user_id, 'displet_user_hashes', true);
				$user_searches[] = $_POST['last_hash'];
				update_user_meta($user_id, 'displet_user_hashes', $user_searches);
			}
		}
		die();
	}

	public function update_viewed_properties($url = '', $address = '', $price = '', $zip = '', $sq_ft = '', $property_id = false){
		$user_id = get_current_user_id();
		if ($user_id != 0 && current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
			$user_searches = get_user_meta($user_id, 'displet_user_properties', true);
			$user_searches[] = array(
				'url' => $url,
				'address' => $address,
				'price' => $price,
				'zip' => $zip,
				'sq_ft' => $sq_ft
			);
			update_user_meta($user_id, 'displet_user_properties', $user_searches);
			if (!empty($api_user_id)) {
				DispletRetsIdxUsersApi::update_user_property_views($api_user_id, $property_id);
			}
		}
	}

	public function delete_users(){
		check_ajax_referer('displet_delete_users_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_users_request'){
			if (!empty($_POST['displet_users'])) {
				$error_occurred = false;
				foreach ($_POST['displet_users'] as $user){
					$deleted[] = wp_delete_user($user);
				}
				foreach ($deleted as $was_deleted) {
					if (!$was_deleted){
						$error_occurred = true;
					}
				}
				if ($error_occurred){
					echo 'There was an error processing your request.';
				}
				else{
					echo 'Succesful Deletion';
				}
			}
			die();
		}
	}

	public function reassign_users(){
		check_ajax_referer('displet_reassign_users_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_reassign_users_request'){
			if (!empty($_POST['displet_users'])) {
				if (!empty($_POST['displet_agent'])) {
					foreach ($_POST['displet_users'] as $user){
						update_user_meta($user, 'displet_agent_id', $_POST['displet_agent']);
						DispletRetsIdxEmails::send_assigned_lead_message($user, $_POST['displet_agent']);
					}
					echo 'Succesful Assignation';
				}
				else{
					echo 'No agent has been selected.';
				}
			}
			die();
		}
	}

	public function save_property(){
		check_ajax_referer('displet_save_property_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_save_property_request'){
			if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
				if (!empty($_POST['type'])) {
					global $displetretsidx_option;
					$user_id = get_current_user_id();
					if ($user_id != 0) {
						$base = get_permalink($displetretsidx_option['property_details_page_id']);
						$url = str_replace($base, '', $_POST['url']);
						$saved_properties = get_user_meta($user_id, 'displet_saved_properties', true);
						$saved_properties[] = array(
							'sysid' => $_POST['sysid'],
							'url' => $url,
							'type' => $_POST['type'],
							'rating' => $_POST['rating'],
							'message' => $_POST['message'],
						);
						update_user_meta($user_id, 'displet_saved_properties', $saved_properties);
						echo 'This property has been saved.';
					}
					else{
						echo 'You are not logged in. Please sign in or create an account and try again.';
					}
				}
				else{
					echo 'Please select Save as favorite, Save as possibility, or Just save notes.';
				}
			}
			else{
				echo 'The current user does not have the capability to save properties.';
			}
			die();
		}
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function delete_saved_properties(){
		check_ajax_referer('displet_delete_properties_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_properties_request'){
			if (!empty($_POST['displet_properties'])) {
				$user_id = get_current_user_id();
				$saved_properties = get_user_meta($user_id, 'displet_saved_properties');
				$delete_properties = $_POST['displet_properties'];
				foreach ($delete_properties as $property_id){
					unset($saved_properties[0][$property_id]);
				}
				$saved_properties[0] = array_values($saved_properties[0]);
				update_user_meta($user_id, 'displet_saved_properties', $saved_properties[0]);
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public function save_search(){
		check_ajax_referer('displet_save_search_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_save_search_request'){
			if (!empty($_POST['hash'])) {
				$user_id = get_current_user_id();
				if (!empty($user_id)) {
					if (current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
						if (!empty($_POST['search_name'])) {
							$saved_searches = get_user_meta($user_id, 'displet_saved_searches', true);
							$name_exists = false;
							if (!empty($saved_searches) && is_array($saved_searches)) {
								foreach ($saved_searches as $saved_search) {
									if ($saved_search['name'] == $_POST['search_name']){
										$name_exists = true;
									}
								}
							}
							if (empty($name_exists)) {
								$this_search = array(
									'hash' => $_POST['hash'],
									'name' => $_POST['search_name']
								);
								$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
								if (!empty($api_user_id)) {
									$saved_search_criteria = self::get_search_criteria_from_hash($_POST['hash']);
									$saved_search_id = DispletRetsIdxUsersApi::create_saved_search($api_user_id, $_POST['search_name'], $saved_search_criteria);
									if (isset($saved_search_id)) {
										$this_search['api_id'] = $saved_search_id;
									}
								}
								$saved_searches[] = $this_search;
								update_user_meta($user_id, 'displet_saved_searches', $saved_searches);
								echo 'Saved Search';
								die();
							}
							else {
								echo 'The search "' . $_POST['search_name'] . '" already exists. Please select a different name.';
							}
						}
						else {
							echo 'Please name the search.';
						}
					}
					else{
						echo 'The current user does not have the capability to save properties.';
					}
				}
				else{
					echo 'Not Logged In';
				}
			}
			else{
				echo 'Please perform a search to save.';
			}
			die();
		}
		
		echo 'We\'re sorry, there was an error processing your request. Please try again.';
		die();
	}

	public function delete_saved_searches(){
		check_ajax_referer('displet_delete_searches_nonce');
		if (isset($_POST['action']) && $_POST['action'] == 'displet_delete_searches_request'){
			if (!empty($_POST['displet_searches'])) {
				$user_id = get_current_user_id();
				$saved_searches = get_user_meta($user_id, 'displet_saved_searches', true);
				$delete_searches = $_POST['displet_searches'];
				foreach ($delete_searches as $search){
					unset($saved_searches[$search['array_index']]);
					if (!empty($search['api_id'])) {
						$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
						DispletRetsIdxUsersApi::delete_saved_search($api_user_id, $search['api_id']);
					}
				}
				$saved_searches = array_values($saved_searches);
				update_user_meta($user_id, 'displet_saved_searches', $saved_searches);
				echo 'Succesful Deletion';
			}
			else{
				echo 'No property was selected for deletion.';
			}
			die();
		}
		else {
			echo 'There was an error processing your request. Please try again.';
		}
		die();
	}

	public function set_referral_cookie(){
		if (empty($_COOKIE['displetretsidx_referring_site'])) {
        	setcookie('displetretsidx_referring_site', $_SERVER['HTTP_REFERER'], 0, '/');
		}
	}

	private function get_search_criteria_from_hash($hash){
		$trimmed_saved_search_criteria = rtrim(ltrim($hash, '#'), '/');
		$saved_search_criteria = str_replace('/', '&', $trimmed_saved_search_criteria);
		return $saved_search_criteria;
	}

	private function get_first_and_last_name($name){
		$names = array();
		preg_match('/[^\s]+/', $name, $first_name_match);
		if (!empty($first_name_match)) {
			$names['first_name'] = $first_name_match[0];
		}
		else{
			$names['first_name'] = $name;
		}
		$names['last_name'] = trim(str_replace($names['first_name'], '', $name));
		return $names;
	}

	private function upgrade_users_to_role_2(){
		$leads = self::get_re_search_users();
		if (!empty($leads)) {
			foreach ($leads as $lead) {
				$name = get_user_meta($lead->ID, 'nickname', true);
				$names = self::get_first_and_last_name($name);
				if (!empty($names)) {		
					wp_update_user(array(
						'ID' => $lead->ID,
						'display_name' => $names['first_name']
					));
					update_user_meta($lead->ID, 'first_name', $names['first_name']);
					update_user_meta($lead->ID, 'last_name', $names['last_name']);
				}
				$api_user_id = get_user_meta($lead->ID, 'displet_api_user_id', true);
				if (empty($api_user_id)) {
					$api_user_id = DispletRetsIdxUsersApi::create_user($lead->user_email, $names['first_name'], $names['last_name']);
					update_user_meta($lead->ID, 'displet_api_user_id', $api_user_id);
				}
				self::send_users_saved_searches_to_api($lead->ID, $api_user_id);
			}
		}
	}

	public function upgrade_users_to_role_3(){
		$leads = self::get_re_search_users();
		if (!empty($leads)) {
			foreach ($leads as $lead) {
				$api_user_id = get_user_meta($lead->ID, 'displet_api_user_id', true);
				$viewed_properties = get_user_meta($lead->ID, 'displet_user_properties', true);
				if (!empty($viewed_properties) && !empty($api_user_id)) {
					foreach ($viewed_properties as $viewed_property) {
						wp_schedule_single_event(time(), 'displetretsidx_user_role_3_cron_jobs', array($viewed_property['url'], $api_user_id));
					}
				}
			}
		}
	}

	public function upgrade_users_to_role_3_cron($url, $api_user_id){
		$property_id = self::get_property_id_from_url($url);
		$model = DispletRetsIdxUrlStructure::create_model(false);
		$model['property_id'] = $property_id;
		DispletRetsIdxPropertyDetails::build_model($model);
		global $displetretsidx_model;
		if (!empty($displetretsidx_model['listings'][0]) && !empty($displetretsidx_model['listings'][0]->id)) {
			wp_schedule_single_event(time(), 'displetretsidx_user_role_3_single_cron_jobs', array($api_user_id, $displetretsidx_model['listings'][0]->id));
		}
	}

	public function upgrade_users_to_role_4(){
		$users = self::get_re_search_users();
		if (!empty($users) && is_array($users)) {
			$i = 0;
			foreach ($users as $user) {
				if (empty($user->displet_api_user_id)){
					$args = array(
						'user_id' => $user->ID,
						'user_email' => $user->user_email,
						'user_first_name' => $user->first_name,
						'user_last_name' => $user->last_name,
					);
					wp_schedule_single_event(time() + 10 + $i * 30, 'displetretsidx_user_role_4_cron_jobs', array($args));
					$i++;
				}
			}
		}
	}

	public function upgrade_users_to_role_4_cron($args){
		$api_user_id = DispletRetsIdxUsersApi::create_user($args['user_email'], $args['user_first_name'], $args['user_last_name']);
		if (!empty($api_user_id)) {
			update_user_meta($args['user_id'], 'displet_api_user_id', $api_user_id);
			self::send_users_saved_searches_to_api($args['user_id'], $api_user_id);
		}
	}

	public function send_users_saved_searches_to_api($user_id, $api_user_id){
		if (!empty($user_id) && !empty($api_user_id)) {
			$saved_searches = get_user_meta($user_id, 'displet_saved_searches', true);
			if (!empty($saved_searches)) {
				$new_saved_searches = array();
				foreach ($saved_searches as $saved_search) {
					if (is_array($saved_search)) {
						$hash = $saved_search['hash'];
						$name = $saved_search['name'];
					}
					else{
						$hash = $saved_search;
						$name = false;
					}
					if (empty($name)) {
						$name = wp_generate_password(12, false);
					}
					$criteria = self::get_search_criteria_from_hash($hash);
					$saved_search_id = DispletRetsIdxUsersApi::create_saved_search($api_user_id, $name, $criteria);
					$new_saved_searches[] = array(
						'hash' => $hash,
						'name' => $name,
						'api_id' => $saved_search_id
					);
				}
				update_user_meta($user_id, 'displet_saved_searches', $new_saved_searches);
			}
		}
	}

	public function get_property_id_from_url($url){
		global $displetretsidx_option;
		if (!empty($url)) {
			$pdp_url = get_permalink($displetretsidx_option['property_details_page_id']);
			$clean_url = str_replace($pdp_url, '', $url);
			preg_match('#[^/]+/[^/]+/[^/]+/([^/]+)#', $clean_url, $matches);
			if (!empty($matches)) {
				return $matches[1];
			}
		}
	}

	public function add_custom_user_role_for_leads(){
		// Making changes in this function requires increasing the user_role_version
		$user_role_version = get_option('displet_rets_idx_user_role_version');
		if (empty($user_role_version) || $user_role_version < 4){
			remove_role('displet_user');
			$capabilities = array(
				'read' => true,
				'displet_save_properties' => true,
				'displet_save_searches' => true
			);
			add_role('displet_user', 'Real Estate Search User', $capabilities);
			if ($user_role_version < 2) {
				self::upgrade_users_to_role_2();
			}
			if ($user_role_version < 3) {
				self::upgrade_users_to_role_3();
			}
			if ($user_role_version < 4) {
				self::upgrade_users_to_role_4();
			}
			update_option('displet_rets_idx_user_role_version', 4);
		}
	}

	public function add_custom_user_role_for_agents(){
		// Making changes in this function requires increasing the user_role_version_admin value
		$user_role_version_agents = get_option('displet_rets_idx_user_role_version_agents');
		if (empty($user_role_version_agents) || $user_role_version_agents < 2){
			remove_role('displet_agent');
			$capabilities = array(
				'read' => true,
				'displet_save_properties' => true,
				'displet_save_searches' => true,
				'displet_view_leads' => true,
				'edit_users' => true
			);
			add_role('displet_agent', 'Displet RE Agent', $capabilities);
			update_option('displet_rets_idx_user_role_version_agents', 2);
		}
	}

	public function add_custom_capabilities_to_admin(){
		// Making changes in this function requires increasing the user_role_version_agents value
		$user_role_version_admin = get_option('displet_rets_idx_user_role_version_admin');
		if (empty($user_role_version_admin) || $user_role_version_admin < 1){
			$role = get_role('administrator');
			$role->add_cap('displet_view_leads');
			update_option('displet_rets_idx_user_role_version_admin', 1);
		}
	}

	public function add_saved_properties_menu(){
		if (!current_user_can('manage_options')) {
			add_menu_page('Saved Properties', 'Saved Properties', 'displet_save_properties', 'displetretsidx_saved_properties', array('DispletRetsIdxUsers', 'add_saved_properties_page'), plugins_url('displetreader-wordpress-plugin/css/images/saved-properties-icon.png'), 71);
		}
	}

	public function add_saved_properties_page(){
		require_once dirname(__FILE__) . '/users/displet-saved-properties-page.php';
	}

	public function add_saved_searches_menu(){
		if (!current_user_can('manage_options')) {
			add_menu_page('Saved Searches', 'Saved Searches', 'displet_save_searches', 'displetretsidx_saved_searches', array('DispletRetsIdxUsers', 'add_saved_searches_page'), plugins_url('displetreader-wordpress-plugin/css/images/saved-searches-icon.png'), 72);
		}
	}

	public function add_saved_searches_page(){
		require_once dirname(__FILE__) . '/users/displet-saved-searches-page.php';
	}

	public function add_search_properties_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			global $displetretsidx_option;
			$href = get_permalink($displetretsidx_option['search_results_page_id']);
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_search_properties_toolbar',
    			'title' => 'Search Properties',
    			'href' => $href
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_search_properties_toolbar_SUB',
    			'title' => 'Search Properties',
    			'href' => $href,
    			'parent' => 'site-name'
    		));
		}
	}

	public function add_saved_properties_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_properties_toolbar',
    			'title' => 'Saved Properties',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_properties')
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_properties_toolbar_sub',
    			'title' => 'Saved Properties',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_properties'),
    			'parent' => 'site-name'
    		));
		}
	}

	public function add_saved_searches_to_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_searches') && !current_user_can('manage_options')) {
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_searches_toolbar',
    			'title' => 'Saved Searches',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_searches')
    		));
    		$wp_admin_bar->add_node(array(
    			'id' => 'displetretsidx_saved_searches_toolbar_sub',
    			'title' => 'Saved Searches',
    			'href' => admin_url('admin.php?page=displetretsidx_saved_searches'),
    			'parent' => 'site-name'
    		));
		}
	}

	public function clean_up_user_toolbar($wp_admin_bar){
		if (current_user_can('displet_save_properties') && !current_user_can('displet_view_leads')) {
    		$wp_admin_bar->remove_node('wp-logo');
    		$wp_admin_bar->remove_node('search');
		}
	}

	public function clean_up_user_dashboard(){
		if (current_user_can('displet_save_properties') && !current_user_can('manage_options')) {
			remove_meta_box('dashboard_primary', 'dashboard', 'side');
			remove_meta_box('dashboard_secondary', 'dashboard', 'side');
		}
	}

	public function send_to_zapier($user_data, $zap){
		global $displetretsidx_option;
		if (!empty($displetretsidx_option['zapier_' . $zap . '_url'])) {
			$request = array(
				'user-agent' => 'Displet',
				'body' => $user_data,
				'referer' => home_url()
			);
			$urls = array_map('trim', explode("\n", $displetretsidx_option['zapier_' . $zap . '_url']));
			foreach ($urls as $url) {
				$post_url = trim($url);
				if (!empty($post_url)) {
					wp_remote_post($post_url, $request);
				}
			}
		}
	}

	public function add_to_agent_page($user){
		if (user_can($user->ID, 'displet_view_leads')) {

			?>
				<h3>Displet Settings</h3>
				<table class="form-table">
					<tr>
						<th><label for="displet_email_signature">Email Signature</label></th>
						<td>
							<textarea name="displet_email_signature" id="displet_email_signature" rows="5" cols="30"><?php echo esc_attr(get_the_author_meta('displet_email_signature', $user->ID)); ?></textarea><br />
							<span class="description">Displayed at the bottom of each outbound email.</span>
						</td>
					</tr>
				</table>
			<?php
		}
	}

	public function save_from_agent_page($user_id){
		$saved = false;
		if (user_can($user_id, 'displet_view_leads') && current_user_can('edit_user', $user_id)) {
			update_user_meta($user_id, 'displet_email_signature', $_POST['displet_email_signature']);
			$saved = true;
		}
		return true;
	}

	public function save_from_user_page($user_id){
		if (user_can($user_id, 'displet_save_properties') && current_user_can('edit_user', $user_id)) {
			$api_user_id = get_user_meta($user_id, 'displet_api_user_id', true);
			if (!empty($api_user_id)) {
				$user_array = array();
				if (!empty($_POST['first_name'])) {
					$user_array['first_name'] = $_POST['first_name'];
				}
				if (!empty($_POST['last_name'])) {
					$user_array['last_name'] = $_POST['last_name'];
				}
				if (!empty($_POST['email'])) {
					$user_array['email'] = $_POST['email'];
				}
				DispletRetsIdxUsersApi::update_user($api_user_id, $user_array);
			}
		}
	}

	public function get_property_view_stats($properties){
		$stats = array(
			'price_average' => '',
			'square_feet_average' => '',
			'zip_mode' => ''
		);
		if (!empty($properties) && is_array($properties)) {
			$price_total = $price_count = $sq_ft_total = $sq_ft_count = 0;
			$properties_zips = array();
			foreach ($properties as $property) {
				if (!empty($property['price']) && intval($property['price']) > 0) {
					$price_total += intval($property['price']);
					$price_count++;
				}
				if (!empty($property['sq_ft']) && intval($property['sq_ft']) > 0) {
					$sq_ft_total += intval($property['sq_ft']);
					$sq_ft_count++;
				}
				if (!empty($property['zip'])) {
					$properties_zips[] = $property['zip'];
				}
			}
			if ($price_count > 0) {
				$stats['price_average'] = '$' . number_format(intval($price_total / $price_count));
			}
			if ($sq_ft_count > 0) {
				$stats['square_feet_average'] = number_format(intval($sq_ft_total / $sq_ft_count));
			}
			if (!empty($properties_zips)) {
				$values = array_count_values($properties_zips); 
				$stats['zip_mode'] = array_search(max($values), $values);
			}
		}
		return $stats;
	}

	private function get_re_search_users(){
		return get_users(array(
			'meta_key' => 'displet_lead',
			'meta_value' => true
		));
	}
}

add_action('wp_ajax_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_registration_request', array('DispletRetsIdxUsers', 'register_user_ajax'));
add_action('wp_ajax_displetretsidx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_nopriv_displetretsidx_user_signon', array('DispletRetsIdxUsers', 'login_user_ajax'));
add_action('wp_ajax_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_nopriv_displet_user_check_request', array('DispletRetsIdxUsers', 'check_user_ajax'));
add_action('wp_ajax_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_nopriv_displet_check_login_request', array('DispletRetsIdxUsers', 'check_login_ajax'));
add_action('wp_ajax_displet_delete_users_request', array('DispletRetsIdxUsers', 'delete_users'));
add_action('wp_ajax_displet_reassign_users_request', array('DispletRetsIdxUsers', 'reassign_users'));
add_action('wp_login', array('DispletRetsIdxUsers', 'update_last_login'), 10, 2);
add_action('wp_ajax_displet_update_searches', array('DispletRetsIdxUsers', 'update_viewed_searches'));
add_action('wp_ajax_nopriv_displet_update_searches', array('DispletRetsIdxUsers', 'update_viewed_searches'));
add_action('wp_ajax_displet_save_property_request', array('DispletRetsIdxUsers', 'save_property'));
add_action('wp_ajax_nopriv_displet_save_property_request', array('DispletRetsIdxUsers', 'save_property'));
add_action('wp_ajax_displet_delete_properties_request', array('DispletRetsIdxUsers', 'delete_saved_properties'));
add_action('wp_ajax_nopriv_displet_delete_properties_request', array('DispletRetsIdxUsers', 'delete_saved_properties'));
add_action('wp_ajax_displet_save_search_request', array('DispletRetsIdxUsers', 'save_search'));
add_action('wp_ajax_nopriv_displet_save_search_request', array('DispletRetsIdxUsers', 'save_search'));
add_action('wp_ajax_displet_delete_searches_request', array('DispletRetsIdxUsers', 'delete_saved_searches'));
add_action('wp_ajax_nopriv_displet_delete_searches_request', array('DispletRetsIdxUsers', 'delete_saved_searches'));
add_action('init', array('DispletRetsIdxUsers', 'set_referral_cookie'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_user_role_for_leads'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_user_role_for_agents'));
add_action('init', array('DispletRetsIdxUsers', 'add_custom_capabilities_to_admin'));
add_action('admin_menu', array('DispletRetsIdxUsers', 'add_saved_properties_menu'));
add_action('admin_menu', array('DispletRetsIdxUsers', 'add_saved_searches_menu'));
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_search_properties_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_saved_properties_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'add_saved_searches_to_toolbar'), 999);
add_action('admin_bar_menu', array('DispletRetsIdxUsers', 'clean_up_user_toolbar'), 9999);
add_action('admin_menu', array('DispletRetsIdxUsers', 'clean_up_user_dashboard'), 9999);
add_action('show_user_profile', array('DispletRetsIdxUsers', 'add_to_agent_page'));
add_action('edit_user_profile', array('DispletRetsIdxUsers', 'add_to_agent_page'));
add_action('personal_options_update', array('DispletRetsIdxUsers', 'save_from_agent_page'));
add_action('edit_user_profile_update', array('DispletRetsIdxUsers', 'save_from_agent_page'));
add_action('personal_options_update', array('DispletRetsIdxUsers', 'save_from_user_page'));
add_action('edit_user_profile_update', array('DispletRetsIdxUsers', 'save_from_user_page'));
add_action('displetretsidx_new_user_cron_jobs', array('DispletRetsIdxUsers', 'new_user_cron_jobs'));
add_action('displetretsidx_user_role_3_cron_jobs', array('DispletRetsIdxUsers', 'upgrade_users_to_role_3_cron'), 10, 2);
add_action('displetretsidx_user_role_3_single_cron_jobs', array('DispletRetsIdxUsersApi', 'update_user_property_views'), 10, 2);
add_action('displetretsidx_user_role_4_cron_jobs', array('DispletRetsIdxUsers', 'upgrade_users_to_role_4_cron'), 10, 1);

?>