<?php

class DispletRetsIdxUtilities{
	/*****************************
	/ String manipulation
	*****************************/

	public function remove_url_prefixes($string){
		return str_replace(array('http://www.', 'https://www.', 'http://', 'https://'), '', $string);
	}

	public function remove_url_suffixes($string){
		if (strpos($string, '/') !== false) {
			return substr($string, 0, strpos($string, '/'));
		}
		return $string;
	}

	public function get_domainless_url($string){
		$unprefixed = self::remove_url_prefixes($string);
		if (strpos($unprefixed, '/') !== false) {
			return substr($unprefixed, strpos($unprefixed, '/'), strlen($unprefixed) - 1);
		}
		return $unprefixed;
	}

	public function add_spaces_after_commas($string){
		$string = preg_replace('/(,$)/', '', $string);
		return preg_replace('/(?<=,)(?!\\s)/', ' ', $string);
	}

	public function get_address_from_property_details_suffix($string){
		if (!empty($string)) {
			preg_match('~([^/]+)/?([^/]*)/?([^/]*)/?([^/]*)/?([^/]*)/?~', $string, $matches);
			if (!empty($matches[5])) {
				return str_replace('-', ' ', $matches[5]);
			}
		}
		return false;
	}
	
	/*****************************
	/ Option related
	*****************************/

	public function is_new_api_key($original_option, $new_option){
		return self::is_option_new('displet_app_key', $original_option, $new_option);
	}

	public function is_new_api_token($original_option, $new_option){
		return self::is_option_new('displet_app_token', $original_option, $new_option);
	}

	public function is_new_email_address($original_option, $new_option){
		return self::is_option_new('email_from_address', $original_option, $new_option);
	}

	public function email_activity_has_changed($original_option, $new_option){
		return self::has_option_changed('email_activity', $original_option, $new_option);
	}

	public function is_option_new($array_key, $original_option, $new_option){
		if (!empty($new_option[$array_key]) && self::has_option_changed($array_key, $original_option, $new_option)) {
			return true;
		}
		return false;
	}

	public function has_option_changed($array_key, $original_option, $new_option){
		if (empty($original_option[$array_key]) || $new_option[$array_key] !== $original_option[$array_key]) {
			return true;
		}
		return false;
	}

	public function get_widget_area($slug, $name){
		if (!function_exists('dynamic_sidebar') || !dynamic_sidebar($slug)) {
			if (current_user_can('manage_options')) {
				echo 'Insert Displet Search Field widgets into the ' . $name . ' widget area on your <a href="' . admin_url('widgets.php') . '">Widgets page</a>.';
			}
		}
	}
	
	/*****************************
	/ Miscellaneous
	*****************************/

	public function is_referral_function($f) {
		$aray = debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS);
		foreach($aray as $a) {	$aa[] = $a["function"];	}
		if(in_array($f,$aa)) { return true; }
		return false;
	}
}

/**
 * External Functions below
 * 
 * Do not change the function name, parameters, or return type/value under any circumstance
 */
	
/*****************************
/ Option related
*****************************/

/**
 * @param: none
 * @return: string
 */
function displetretsidx_get_api_key(){
	//$option = DispletRetsIdxSettings::get_option();
	global $displetretsidx_option;
	return $displetretsidx_option['displet_app_key'];
}

/**
 * @param: none
 * @return: string
 */
function displetretsidx_get_property_details_page_id(){
	//$option = DispletRetsIdxSettings::get_option();
	global $displetretsidx_option;
	return $displetretsidx_option['property_details_page_id'];
}

/**
 * @param: none
 * @return: string
 */
function displetretsidx_get_search_results_page_id(){
	//$option = DispletRetsIdxSettings::get_option();
	global $displetretsidx_option;
	return $displetretsidx_option['search_results_page_id'];
}

/**
 * @param: $setting: string(min_price, max_price, price_increment, beds_min, beds_max, baths_min, baths_max, min_square_feet, max_square_feet, square_feet_increment, min_acres, max_acres, or acres_increment)
 * @return: success: integer
 * @return: failure: false
 */
function displetretsidx_get_search_value($setting){
	//$option = DispletRetsIdxSettings::get_option();
	global $displetretsidx_option;
	if (isset($displetretsidx_option['search_values_' . $setting])) {
		return intval($displetretsidx_option['search_values_' . $setting]);
	}
	return false;
}

/**
 * @param: $field: string(area_mls_defined, city, property_type or school_district or status)
 * @return: success: array
 * @return: failure: false
 */
function displetretsidx_get_field_options($field){
	$field_option = DispletRetsIdxSettings::get_option('fields');
	if (isset($field_option[$field])) {
		return $field_option[$field];
	}
	return false;
}
	
/*****************************
/ Widget related
*****************************/

/**
 * @param: none
 * @return: print: dynamic_sidebar
 */
function displetretsidx_get_mobile_search_form(){
	DispletRetsIdxUtilities::get_widget_area('displetretsidx-search-form-mobile', 'Displet Mobile Search Form');
}

/**
 * @param: none
 * @return: print: dynamic_sidebar
 */
function displetretsidx_get_mobile_quick_search_form(){
	DispletRetsIdxUtilities::get_widget_area('displetretsidx-quick-search-form-mobile', 'Displet Mobile Quick Search Form');
}

/**
 * @param: none
 * @return: print: dynamic_sidebar
 */
function displetretsidx_get_mobile_contact_page_top(){
	DispletRetsIdxUtilities::get_widget_area('displetretsidx-mobile-contact-top', 'Displet Mobile Contact Page Top');
}

/**
 * @param: none
 * @return: print: dynamic_sidebar
 */
function displetretsidx_get_mobile_contact_page_bottom(){
	DispletRetsIdxUtilities::get_widget_area('displetretsidx-mobile-contact-bottom', 'Displet Mobile Contact Page Bottom');
}
	
/*****************************
/ Menu related
*****************************/

/**
 * @param: none
 * @return: print: wp_nav_menu
 */
function displetretsidx_get_mobile_menu(){
	wp_nav_menu(array('theme_location' => 'displetretsidx_mobile_menu'));
}
	
/*****************************
/ Template related
*****************************/

/**
 * @param: $filename: string (without .php)
 * @return: print: include_once
 */
function displetretsidx_get_template_part($filename){
	include_once(DispletRetsIdxLoad::get_template_path($filename . '.php'));
}
	
/*****************************
/ Listings related
*****************************/

/**
 * @param: $criteria: array('field' => 'value'); $num_listings: integer, 1-50
 * @return: success - array: $results['listings'] = object(stdClass), $results['meta'] = object(stdClass), $results['query_url'] = string
 * @return: failure - boolean: false
 */
function displetretsidx_get_listings($criteria, $num_listings = false){
	$model = DispletRetsIdxModel::get_model();
	if (!empty($criteria) && is_array($criteria)) {
		$shortcode_translations = DispletRetsIdxResidentials::get_shortcode_translations(false);
		$shortcode_translations_reverse = array_flip($shortcode_translations);
		foreach ($criteria as $key => $value) {
			if (!empty($shortcode_translations_reverse[$key])) {
				$model[$shortcode_translations_reverse[$key]] = $value;
			}
		}
	}
	if (!empty($num_listings) && is_numeric($num_listings)) {
		$the_count = intval($num_listings);
		if ($the_count > 0 && $the_count < 50) {
			$model['listings_per_page'] = $the_count;
		}
	}
	if (!empty($criteria['return_fields'])) {
		$model['return_fields'] = $criteria['return_fields'];
	}
	if (!empty($criteria['page']) && is_numeric($criteria['page'])) {
		$the_page = intval($criteria['page']);
		if ($the_page > 0) {
			$model['page'] = $the_page;
		}
	}
	$model = DispletRetsIdxResidentialsApi::get_listings($model, false);
	if (!empty($model['listings'])) {
		$results = array(
			'listings' => $model['listings'],
			'meta' => $model['meta'],
			'query_url' => $model['query_url']
		);
		return $results;
	}
	return false;
}

/**
 * @param: $criteria: array('field' => 'value')
 * @return: success - integer
 * @return: failure - boolean: false
 */
function displetretsidx_get_count($criteria = false){
	$model = DispletRetsIdxModel::get_model();
	if (!empty($criteria) && is_array($criteria)) {
		$shortcode_translations = DispletRetsIdxResidentials::get_shortcode_translations(false);
		$shortcode_translations_reverse = array_flip($shortcode_translations);
		foreach ($criteria as $key => $value) {
			if (!empty($shortcode_translations_reverse[$key])) {
				$model[$shortcode_translations_reverse[$key]] = $value;
			}
		}
	}
	$count = DispletRetsIdxResidentialsApi::get_count($model);
	if (!empty($count)) {
		return $count;
	}
	return false;
}

?>